/**
 * Convert a single file-input element into a 'multiple' input list
 *
 * Usage:
 *
 *   1. Create a file input element (no name)
 *      eg. <input type="file" id="first_file_element">
 *
 *   2. Create a DIV for the output to be written to
 *      eg. <div id="files_list"></div>
 *
 *   3. Instantiate a MultiSelector object, passing in the DIV and an (optional) maximum number of files
 *      eg. var multi_selector = new MultiSelector( document.getElementById( 'files_list' ), 3 );
 *
 *   4. Add the first element
 *      eg. multi_selector.addElement( document.getElementById( 'first_file_element' ) );
 *
 *   5. That's it.
 *
 *   You might (will) want to play around with the addListRow() method to make the output prettier.
 *
 *   You might also want to change the line 
 *       element.name = 'file_' + this.count;
 *   ...to a naming convention that makes more sense to you.
 * 
 * Licence:
 *   Use this however/wherever you like, just don't blame me if it breaks anything.
 *
 * Credit:
 *   If you're nice, you'll leave this bit:
 *  
 *   Class by Stickman -- http://www.the-stickman.com
 *      with thanks to:
 *      [for Safari fixes]
 *         Luis Torrefranca -- http://www.law.pitt.edu
 *         and
 *         Shawn Parker & John Pennypacker -- http://www.fuzzycoconut.com
 *      [for duplicate name bug]
 *         'neal'
 */
function MultiSelector(table, max) {

    	var str1;
	var cell;

	// Where to write the list
	this.table = table;
	// How many elements?
	this.count = 0;
	// How many elements?
	this.id = 0;
	// Is there a maximum?
	if( max ){
		this.max = max;
	} else {
		this.max = -1;
	};

	/* We assume that the first row has file input element */
	/*
	for (i = 0; i < table.rows[0].cells.length; i++) {

	    	cell = table.rows[0].cells[i];

		for (j = 0; j < cell.childNodes.length; j++) {

			if (cell.childNodes[j].tagName == 'INPUT' && cell.childNodes[j].type == 'file') {
				addElement(cell.childNodes[j])
				break;
			}
		}
	}
*/	

	/**
	 * Add a new file input element
	 */
	this.addElement = function(element) {

		if (this.id == 0) {
			this.name = element.name;
		}

		// Make sure it's a file input element
		if( element.tagName == 'INPUT' && element.type == 'file' ){

			// Element name -- what number am I?
			element.name = this.name + this.id++;

			// Add reference to this object
			element.multi_selector = this;

			// What to do when a file is selected
			element.onchange = function() {

				// New file input
				var new_element = document.createElement( 'input' );
				new_element.type = 'file';
				new_element.size = element.size;

				// Add new element
				this.parentNode.insertBefore( new_element, this );

				// Apply 'update' to element
				this.multi_selector.addElement( new_element );

				// Update list
				this.multi_selector.addListRow( this );

				// Hide this: we can't use display:none because Safari doesn't like it
				this.style.position = 'absolute';
				this.style.left = '-1000px';

			};
			// If we've reached maximum number, disable input element
			if( this.max != -1 && this.count >= this.max ){
				element.disabled = true;
			};

			// File element counter
			this.count++;
			// Most recent element
			this.current_element = element;
			
		} else {
			// This can only be applied to file input elements!
			alert( 'Error: not a file input element' );
		};
	};

	/**
	 * Add a new row to the list of files
	 */
	this.addListRow = function( element ) {

		var table = element.multi_selector.table;
		var rowindex = table.rows.length;
		var new_row = table.insertRow(rowindex);
		var new_col1 = new_row.insertCell(0);
		var new_col2 = new_row.insertCell(1);
		var button = document.createElement("input");

		table.rows[0].cells[0].rowSpan++;

		button.type = "button";
		button.value = "Delete...";

		/* Reference */
		button.table = table;
		button.row = new_row;
		button.element = element;

		/* Delete function */
		button.onclick = function() {

			/* Remove file input */
			this.element.parentNode.removeChild(this.element);

			/* Remove the table row */
			this.table.deleteRow(this.row.rowIndex);

			/* Decrease the span */
			table.rows[0].cells[0].rowSpan--;
			this.element.multi_selector.count--;
			this.element.multi_selector.current_element.disabled = false;

			return false;
		}

		new_col1.innerHTML = element.value;

		//new_col2.innerHTML = "<input type='button' value='Delete...' onclick=";
		new_col2.appendChild(button);
	}
};
