/*
 *  verizon style dropdown
 *
 *  ref: http://vzonline.epdigitalsystems.com/global/drop-downs
 *  note: select - option => link(a) / menu(ul) - item(li)
 *
 */

$(document).ready(function() {
    VZWDropdown.initialize();
});

var VZWDropdown = {

    selects     : null,
    dropdowns   : null,
    inited      : false,
    addIEShadow : false,

    initialize : function()
    {
        if (this.inited)
            return;

        this.getSelects();
        this.hideSelects();
        this.createDropdowns();
        this.initEvents();

        if (typeof $('div').css('boxShadow') === 'undefined') {
            this.addIEShadow = true;
        }
        this.inited = true;
        return this;
    },

    reinitilize : function()
    {
        this.inited = false;
        this.initialize();
    },

    recreate : function(selectId)
    {
        $('#' + selectId + 'Link ~ .dropdownMenuShadow').remove();
        $('#' + selectId + 'Link').remove();
        $('#' + selectId + 'Menu').remove();
        $('#' + selectId).data('dropdown', '');
        this.reinitilize();
    },

    getSelects : function()
    {
        this.selects = $('select.dropdown');
    },

    hideSelects : function()
    {
        this.selects.css({ position: 'absolute', left: '-9999px', top: '-9999px' });
    },

    createDropdowns : function()
    {
        var self          = this,
            selects       = this.selects,
            dropdowns     = [],
            selectedClass = '',
            select, options, option, selectedOption, forceMenuWidth, link, menu, item;

        for (var i = selects.length; i--; ) {
            select = selects[i];
            if ($(select).data('dropdown') === 'created') {
                continue;
            }
            selectedOption = $('#' + select.id + ' option:selected')[0] || select.options[0];
            forceMenuWidth = $(select).data('dropdown-width') ? $(select).data('dropdown-width') : 0;

            // create trigger link
            link = $('<a>', { id: select.id + 'Link', href: 'javascript: void(0);', 'class': 'dropdownLink', 'rel': select.id, html: '<span>' + selectedOption.text + '<\/span>' });
            $(select).after(link);
            link.on('click', function(e) {
                e.stopPropagation();
                self.toggleDropdown(this);
            });
            link.on('focus', function(e) {
                e.stopPropagation();
                self.focusDropdown(this);
            });

            // create dropdown menu
            menu = $('<ul>', { 'class': 'dropdownMenu', id: select.id + 'Menu' });

            // populate dropdown menu
            options = $('#' + select.id + ' option');
            for (var j = 0, k = options.length; j < k; ++j) {
                option = options[j];
                selectedClass = (option === selectedOption) ? 'selected' : '';
                item = $('<li>', {html: '<a href="javascript: void(0);" class="' + selectedClass + '" ref="' + option.value + '">' + option.text + '</a>'});
                item.data('selectId', select.id);
                item.data('optionText', option.text);
                item.data('optionValue', option.value);
                menu.append(item);
                item.on('click', function(e) {
                    e.preventDefault();
                    self.selectOption(this);
                });
            }

            $(link).after(menu);
            menu.css({ position: 'absolute', top: '-9999em', left: '-9999em', visibility: 'hidden', display: 'block' });
            // for handling the case that menu is inside a hidden element.
            menu_width = (forceMenuWidth ? forceMenuWidth : menu.prop('offsetWidth')) - 2 + 'px';
            link.css('width', menu_width);
            menu.css({ visibility: 'visible', display: 'none' });
            menu.find('a').on('blur', function(e) {
                var links = $(this).parents('ul').find('a');
                if (this === links[links.length - 1]) {
                    self.hideDropdown();
                }
            });
            link.after($('<div>', { 'class': 'dropdownMenuShadow' }));
            $(select).data('dropdown', 'created');
        }
    },

    toggleDropdown : function(link)
    {
        var self = this,
            menu = $('#' + link.rel + 'Menu')[0];
        if (menu) {
            // close any other open menus
            $('ul.dropdownMenu').each(function(obj) {
                if (obj !== menu)
                    self.hideDropdown(obj);
            });
            if ($(menu).css('display') === 'none') {
                this.showDropdown(menu, link);
            } else {
                this.hideDropdown(menu);
            }
        }
    },

    showDropdown : function(menu, link)
    {
        var pos = $(link).position();
        $(menu).css({ top: pos.top, left: pos.left, display: 'block' });
        if (this.addIEShadow) {
            $(menu).prev('.dropdownMenuShadow').css({ top: pos.top - 4, left: pos.left - 4, width: menu.offsetWidth, height: menu.offsetHeight, display: 'block' });
        }
    },

    hideDropdown : function(menu)
    {
        if (menu) {
            $(menu).css('display', 'none');
            $(menu).prev('.dropdownMenuShadow').css('display', 'none');
        } else {
            $('ul.dropdownMenu').css('display', 'none');
            $('div.dropdownMenuShadow').css('display', 'none');
        }
    },

    focusDropdown : function(link)
    {
        var self = this,
            menu = $('#' + link.rel + 'Menu')[0];

        setTimeout(function() { self.showDropdown(menu, link); }, 150); /* add a delay because a click will first fire a focus event so then the click will be fired on the document, closing the dropdown */
    },

    selectOption : function(item)
    {
        var selectId    = $(item).data('selectId'),
            optionText  = $(item).data('optionText'),
            optionValue = $(item).data('optionValue'),
            select      = $('#' + selectId)[0],
            menu        = $('#' + selectId + 'Menu')[0],
            link        = $('#' + selectId + 'Link')[0],
            originalOption = $('#' + selectId + ' option[value=' + optionValue + ']')[0];

        $(link).html('<span>' + optionText + '</span>');
        $(menu).find('a').removeClass('selected');
        $(item).find('a').addClass('selected');
        $(originalOption).prop('selected', true);

        // fire events
        $(originalOption).trigger('click');
        $(select).trigger('change');
    },

    /* to set dropdown */
    setOption : function(selectId, optionValue)
    {
        var link = $('#' + selectId + 'Link')[0],
            menu = $('#' + selectId + 'Menu')[0],
            item_link = $('#' + selectId + 'Menu a[ref=' + optionValue + ']')[0],
            optionText = $('#' + selectId + ' option[value=' + optionValue + ']')[0].text;
        $(link).html('<span>' + optionText + '</span>');
        $(menu).find('a').removeClass('selected');
        $(item_link).addClass('selected');
        $('#' + selectId).val(optionValue);
    },

    initEvents : function()
    {
        var self = this;

        $(document).on('click', function(e) {
            $('ul.dropdownMenu').each(function(item) { self.hideDropdown(item); });
        });
    }
};
